﻿Import("System.Windows.Application");
Import("System.Windows.Controls.UserControl");

function setContext(thisObj, func) {
  var d = function(){return func.apply(thisObj, arguments);};
  d.thisObj = thisObj;
  d.func = func;
  return d;
}

function App() {
  Application.Current.Debug = true;
  Application.Current.ReportUnhandledErrors = true;
  Application.Current.ErrorTargetID = "errorLocation";

  this.scene = Application.Current.LoadRootVisual(new UserControl(), "app.xaml");
  this.classes = LoadModule("classes");

  this.xbb = new Array();
  this.N = 16;
  this.framesCount = 0;
  this.lastFrame = 0;
  this.lastTime = Date.UTC();
  this.isRunning = true;
  
  this.parentCanvas = this.scene.FindName("parentCanvas");
  this.timer = this.scene.FindName("timer");
  this.fpsTimer = this.scene.FindName("fps_timer");
  this.fps = this.scene.FindName("fps");
  
  this.numballs = this.scene.FindName("numballs");
  this.resize = this.scene.FindName("resize");
  
  this.scene.FindName("stopButton").Click += setContext(this, this.stop);
  this.scene.FindName("startButton").Click += setContext(this, this.start);
  var handler = setContext(this, this.change);
  this.numballs.SelectionChanged += handler;
  this.resize.Checked += handler;
  this.resize.Unchecked += handler;
}
App.prototype = {
  loaded: function(s, e) {
    for (var i = 0; i < this.N; i++) {
      var ball = new this.classes.UIBall();
      this.parentCanvas.Children.Add(ball.content);
      this.xbb.push(ball);
    }
    this.timer.Completed += setContext(this, this.moveTick);
    this.timer.Begin();
    this.fpsTimer.Completed += setContext(this, this.moveUpdateFPS);
    this.fpsTimer.Begin();
  },
  stop: function() {
    if (!this.isRunning) {
      return;
    }
    this.timer.Stop();
    this.fpsTimer.Stop();
    this.isRunning = false;
  },
  start: function() {
    if (this.isRunning) {
      return;
    }
    this.isRunning = true;
    this.timer.Begin();
    this.fpsTimer.Begin();
  },
  change: function(s, e) {
    var n = Number(this.numballs.SelectedItem.Content);
    var r = this.resize.IsChecked;
    this.startN(n, r);
  },
  startN: function(n, r) {
    if (this.isRunning) {
      this.stop(null);
    }
    for (var i = 0; i < this.N; i++) {
      this.parentCanvas.Children.Remove(this.xbb[i].content);
    }
    this.classes.Model.ballRadius = r ? (26.0 * 4 / Math.sqrt(n)) : 26.0;
    this.N = n;
    this.xbb = new Array();
    for (var i = 0; i < this.N; i++) {
      var ball = new this.classes.UIBall();
      this.fps.Parent.Children.Add(ball.content);
      this.xbb.push(ball);
    }
    this.isRunning = true;
    this.timer.Begin();
    this.fpsTimer.Begin();
  },
  moveTick: function(s, e) {
    this.framesCount++;
    for(var i = 0; i < this.N; i++) {
      this.xbb[i].move();
    }
    for (var i = 0; i < this.N; i++) {
      for (var j = i + 1; j < this.N; j++) {
        this.xbb[i].doCollide(this.xbb[j]);
      }
    }
    if (this.isRunning) {
      this.timer.Begin();
    }
  },
  moveUpdateFPS: function(s, e) {
    var now = Date.UTC();
    var dt = now - this.lastTime;
    var cFps = (this.framesCount - this.lastFrame) * 1000 / dt;
    this.fps.Text = cFps + " fps";
    this.lastFrame = this.framesCount;
    this.lastTime = now;
    if (this.isRunning) {
      this.fpsTimer.Begin();
    }
  }
};

app = new App();
app.scene.Loaded += setContext(app, app.loaded);
